// ../code-standard/dist/index.mjs
var CDN = "https://testingcf.jsdelivr.net";
var textarea;
if (typeof document === "object") {
  textarea = document.createElement("textarea");
}
var getObject = (key) => JSON.parse(String(localStorage.getItem(key)));
var setObject = (key, value) => {
  localStorage.setItem(key, JSON.stringify(value));
};
var parseVersion = (version) => version.split(".", 3).map(Number);
var compareVersion = (version, baseVersion) => {
  const [major, minor] = parseVersion(version), [baseMajor, baseMinor] = parseVersion(baseVersion);
  return major > baseMajor || major === baseMajor && minor >= baseMinor;
};

// src/static.ts
var getStaticMwConfig = ({
  variable,
  parserFunction: [p0, p1, ...p2],
  protocol,
  nsid,
  variants,
  redirection,
  ext,
  doubleUnderscore: [d0, d1, d2, d3],
  img
}, modes) => ({
  tags: Object.fromEntries(ext.map((s) => [s, true])),
  tagModes: modes,
  doubleUnderscore: [
    Object.fromEntries((d2 && d0.length === 0 ? Object.keys(d2) : d0).map((s) => [`__${s}__`, true])),
    Object.fromEntries((d3 && d1.length === 0 ? Object.keys(d3) : d1).map((s) => [`__${s}__`, true]))
  ],
  variableIDs: variable,
  functionSynonyms: [
    {
      ...p0,
      ...Object.fromEntries(p2.flat().map((s) => [s, s]))
    },
    Array.isArray(p1) ? Object.fromEntries(p1.map((s) => [s, s.toLowerCase()])) : { ...p1 }
  ],
  urlProtocols: `${protocol}|//`,
  nsid,
  img: Object.fromEntries(Object.entries(img).map(([k, v]) => [k, `img_${v}`])),
  variants,
  redirection
});

// mw/config.ts
var _a;
var ALL_SETTINGS_CACHE = (_a = getObject("InPageEditMwConfig")) != null ? _a : {};
var SITE_ID = typeof mw === "object" ? mw.config.get("wgServerName") + mw.config.get("wgScriptPath") : location.origin;
var SITE_SETTINGS = ALL_SETTINGS_CACHE[SITE_ID];
var VALID = Number(SITE_SETTINGS == null ? void 0 : SITE_SETTINGS.time) > Date.now() - 86400 * 1e3 * 30;
var getConfig = (magicWords, rule, flip) => Object.fromEntries(
  magicWords.filter(rule).filter(({ "case-sensitive": i }) => i !== flip).flatMap(({ aliases, name, "case-sensitive": i }) => aliases.map((alias) => ({
    alias: (i ? alias : alias.toLowerCase()).replace(/:$/u, ""),
    name
  }))).map(({ alias, name }) => [alias, name])
);
var getConfigPair = (magicWords, rule) => [true, false].map((bool) => getConfig(magicWords, rule, bool));
var setConfig = (config) => {
  mw.config.set("extCodeMirrorConfig", config);
};
var getMwConfig = async (modes) => {
  var _a2;
  if (mw.loader.getState("ext.CodeMirror") !== null && !VALID) {
    await mw.loader.using(mw.loader.getState("ext.CodeMirror.data") ? "ext.CodeMirror.data" : "ext.CodeMirror");
  }
  let config = mw.config.get("extCodeMirrorConfig");
  if (!config && VALID) {
    ({ config } = SITE_SETTINGS);
    setConfig(config);
  }
  const isIPE = config && Object.values(config.functionSynonyms[0]).includes(true), nsid = mw.config.get("wgNamespaceIds");
  if ((config == null ? void 0 : config.img) && config.redirection && config.variants && config.variableIDs && !isIPE) {
    config.urlProtocols = config.urlProtocols.replace(/\\:/gu, ":");
    config.tagModes = modes;
    return { ...config, nsid };
  } else if (location.hostname.endsWith(".moegirl.org.cn")) {
    const parserConfig = await (await fetch(
      `${CDN}/npm/wikiparser-node/config/moegirl.json`
    )).json();
    setObject("wikilintConfig", parserConfig);
    config = getStaticMwConfig(parserConfig, modes);
  } else {
    await mw.loader.using("mediawiki.api");
    const { query: { general: { variants }, magicwords, extensiontags, functionhooks, variables } } = await new mw.Api().get({
      meta: "siteinfo",
      siprop: [
        "general",
        "magicwords",
        ...config && !isIPE ? [] : ["extensiontags", "functionhooks"],
        ...(config == null ? void 0 : config.variableIDs) && !isIPE ? [] : ["variables"]
      ],
      formatversion: "2"
    });
    const others = /* @__PURE__ */ new Set(["msg", "raw", "msgnw", "subst", "safesubst"]);
    if (config && !isIPE) {
      const { functionSynonyms: [insensitive] } = config;
      if (!("subst" in insensitive)) {
        Object.assign(insensitive, getConfig(magicwords, ({ name }) => others.has(name)));
      }
    } else {
      const functions = /* @__PURE__ */ new Set([
        ...functionhooks,
        ...variables,
        ...others
      ]);
      config = {
        tags: Object.fromEntries(extensiontags.map((tag) => [tag.slice(1, -1), true])),
        functionSynonyms: getConfigPair(magicwords, ({ name }) => functions.has(name)),
        doubleUnderscore: getConfigPair(
          magicwords,
          ({ aliases }) => aliases.some((alias) => /^__.+__$/u.test(alias))
        )
      };
    }
    config.tagModes = modes;
    config.img = getConfig(magicwords, ({ name }) => name.startsWith("img_"));
    config.variants = variants ? variants.map(({ code }) => code) : [];
    config.redirection = magicwords.find(({ name }) => name === "redirect").aliases;
    config.urlProtocols = mw.config.get("wgUrlProtocols").replace(/\\:/gu, ":");
    (_a2 = config.variableIDs) != null ? _a2 : config.variableIDs = variables;
  }
  setConfig(config);
  ALL_SETTINGS_CACHE[SITE_ID] = { config, time: Date.now() };
  setObject("InPageEditMwConfig", ALL_SETTINGS_CACHE);
  return { ...config, nsid };
};
var getParserConfig = (minConfig, mwConfig) => {
  let config = getObject("wikilintConfig");
  if (config) {
    return config;
  }
  const { tags, nsid, doubleUnderscore, variants, urlProtocols, redirection, functionSynonyms, img } = mwConfig, [insensitive, sensitive] = functionSynonyms;
  config = {
    ...minConfig,
    ext: Object.keys(tags),
    namespaces: mw.config.get("wgFormattedNamespaces"),
    nsid,
    doubleUnderscore: doubleUnderscore.map(
      (obj) => Object.keys(obj).map((s) => s.slice(2, -2))
    ),
    variants,
    protocol: urlProtocols.replace(/\|\\?\/\\?\//u, ""),
    redirection: redirection != null ? redirection : minConfig.redirection
  };
  if (location.hostname.endsWith(".moegirl.org.cn")) {
    config.html[2].push("img");
  }
  config.parserFunction[0] = insensitive;
  if (mw.loader.getState("ext.CodeMirror") === null) {
    for (const [key, val] of Object.entries(insensitive)) {
      if (!key.startsWith("#")) {
        config.parserFunction[0][`#${key}`] = val;
      }
    }
  }
  config.parserFunction[1] = "wikiparser" in globalThis && compareVersion(wikiparse.version, "1.15") && !Object.values(sensitive).includes(true) ? { ...sensitive, "=": "=" } : [...Object.keys(sensitive), "="];
  for (const [key, val] of Object.entries(img)) {
    config.img[key] = val.slice(4);
  }
  return config;
};
export {
  getMwConfig,
  getParserConfig
};
